class Lotties:
    site_analytics_dashboard = (
        "https://assets1.lottiefiles.com/packages/lf20_i2eyukor.json"
    )
    analytics_people = "https://assets7.lottiefiles.com/packages/lf20_dews3j6m.json"
    green_bi_dashboard = "https://assets8.lottiefiles.com/packages/lf20_mcvtkrvc.json"
    notebook_browser_analytics = (
        "https://assets6.lottiefiles.com/packages/lf20_4ejZJA.json"
    )
    metaverse_explorer = (
        "https://assets3.lottiefiles.com/private_files/lf30_fhbalxer.json"
    )
    loading = "https://assets7.lottiefiles.com/packages/lf20_poqmycwy.json"
    loading_pacman = "https://assets2.lottiefiles.com/packages/lf20_lit5uqwc.json"
